//
//  PhotoSource.m
//  FunnyPics
//
//  Created by Nick Dowell on 11/04/2011.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "PhotoSource.h"
#import "Photo.h"

#import "extThree20XML/TTURLXMLResponse.h"

#define TRACE() NSLog(@"%s", __func__)

static NSString * const kRSSFeedAddress = @"http://www.reddit.com/r/funny/new.rss";

@implementation PhotoSource

@synthesize title			= _title;
@synthesize numberOfPhotos	= _numberOfPhotos;
@synthesize maxPhotoIndex	= _maxPhotoIndex;

#pragma mark @protocol TTPhotoSource

- (id<TTPhoto>)photoAtIndex:(NSInteger)index
{
	NSLog(@"%s:%d", __func__, index);
	
	// Note:
	// - this method is hot (called continuously while scrolling)
	// - must return nil to make the controller call load:more:
	
	if (index >= [_items count])
		return nil;
	
	return [_items objectAtIndex:index];
}

#pragma mark @protocol TTModel

/**
 * Indicates that the data has been loaded.
 *
 * Default implementation returns YES.
 */
- (BOOL)isLoaded
{
	TRACE();
	if ([_items count])
		return YES;
	return NO;
}

/**
 * Indicates that the data is in the process of loading.
 *
 * Default implementation returns NO.
 */
- (BOOL)isLoading
{
	TRACE();
	return _request.isLoading;
}

/**
 * Indicates that the data is in the process of loading additional data.
 *
 * Default implementation returns NO.
 */
- (BOOL)isLoadingMore
{
	TRACE();
	return NO;
}

/**
 * Indicates that the model is of date and should be reloaded as soon as possible.
 *
 * Default implementation returns NO.
 */
- (BOOL)isOutdated
{
	TRACE();
	
	if (!_lastCheckDate)
		return YES;
	
	if ([[NSDate date] timeIntervalSinceDate:_lastCheckDate] > 30)
		return YES;
	
	return NO;
}

/**
 * Loads the model.
 *
 * Default implementation does nothing.
 */
- (void)load:(TTURLRequestCachePolicy)cachePolicy more:(BOOL)more
{
	TRACE();
	
	TTURLXMLResponse *response = [[TTURLXMLResponse alloc] init];
	response.isRssFeed = YES;
	
	TTURLRequest *request = [[[TTURLRequest alloc] initWithURL:kRSSFeedAddress delegate:self] autorelease];
	request.cachePolicy = cachePolicy;
	request.response = response;
	[response release];
	
	[request send];
}

/**
 * Cancels a load that is in progress.
 *
 * Default implementation does nothing.
 */
- (void)cancel
{
	TRACE();
	[_request cancel];
}

/**
 * Invalidates data stored in the cache or optionally erases it.
 *
 * Default implementation does nothing.
 */
- (void)invalidate:(BOOL)erase
{
	TRACE();
}

#pragma mark @protocol TTURLRequestDelegate

/**
 * The request has begun loading.
 *
 * This method will not be called if the data is loaded immediately from the cache.
 * @see requestDidFinishLoad:
 */
- (void)requestDidStartLoad:(TTURLRequest*)request
{
	TRACE();
	_request = [request retain];
}

/**
 * The request has loaded some more data.
 *
 * Check the totalBytesLoaded and totalBytesExpected properties for details.
 */
- (void)requestDidUploadData:(TTURLRequest*)request
{
	TRACE();
}

/**
 * The request has loaded data and been processed into a response.
 *
 * If the request is served from the cache, this is the only delegate method that will be called.
 */
- (void)requestDidFinishLoad:(TTURLRequest*)request
{
	TRACE();
	
	TTURLXMLResponse *response = request.response;
	NSArray *items = [response.rootObject valueForKeyPath:@"channel.item"];
	
	if (items) {
		
		[_lastCheckDate release];
		_lastCheckDate = [[NSDate date] retain];
		
		if (!_items)
			_items = [[NSMutableArray alloc] init];
		
		for (NSDictionary *item in items) {
			NSString *description = [item valueForKeyPath:@"description.___Entity_Value___"];
			NSRange extensionRange = {0};
			if ((extensionRange = [description rangeOfString:@"jpg"]).length ||
				(extensionRange = [description rangeOfString:@"png"]).length ||
				(extensionRange = [description rangeOfString:@"gif"]).length ){
				NSRange urlStartRange = [description rangeOfString:@"http://" options:NSBackwardsSearch range:NSMakeRange(0, extensionRange.length + extensionRange.location)];
				NSString *urlString = [description substringWithRange:NSMakeRange(urlStartRange.location, extensionRange.length + extensionRange.location - urlStartRange.location)];
				Photo *photo = [[Photo alloc] init];
				photo.urlString = urlString;
				if ([_items containsObject:photo] == NO) {
					[_items addObject:photo];
					NSLog(@"Added image: %@", photo.urlString);
				}
				[photo release];
			}
		}

		_numberOfPhotos = [_items count];
		_maxPhotoIndex = [_items count];
	}
	
	[_request release];
	_request = nil;
	
	[self didFinishLoad];
}

/**
 * Allows delegate to handle any authentication challenges.
 */
- (void)request:(TTURLRequest*)request didReceiveAuthenticationChallenge:(NSURLAuthenticationChallenge*)challenge
{
	TRACE();
}

/**
 * The request failed to load.
 */
- (void)request:(TTURLRequest*)request didFailLoadWithError:(NSError*)error
{
	TRACE();
	[_request release];
	_request = nil;
	[self didFailLoadWithError:error];
}

/**
 * The request was canceled.
 */
- (void)requestDidCancelLoad:(TTURLRequest*)request
{
	TRACE();
	[_request release];
	_request = nil;
	[self didCancelLoad];
}

@end
